/*
 * MoveTheBlock - upside down tetris
 * code: PRP + DeepSeek
 * licence: Public Domain
 * IDE: Geany
 * Gdańsk - 2025-01-26
 */

#include <SDL2/SDL.h>
#include <iostream>
#include <vector>
#include <cstdlib>
#include <ctime>
#include <thread>
#include <chrono>

const int SCREEN_WIDTH = 720;
const int SCREEN_HEIGHT = 720;
const int BLOCK_SIZE = 90;
const int GRID_WIDTH = SCREEN_WIDTH / BLOCK_SIZE;
const int GRID_HEIGHT = SCREEN_HEIGHT / BLOCK_SIZE;

struct Block {
	SDL_Rect rect;
	int x, y;
	int gridX, gridY;
    int length;
    int index;
    bool selected;
    bool isDragging;
};

std::vector<Block> blocks;
Block * lastDisplay = nullptr;
bool moved = false;


void gameLogic() {
    while (true) {
		//std::cout << "Logic thread..." << std::endl;
        std::this_thread::sleep_for(std::chrono::milliseconds(500));
        moved = false;
    }
}


void generateBlocks() {
    blocks.clear();
    int index = 0;
    for (int gridY = 0; gridY < GRID_HEIGHT; ++gridY) {
        int gridX = 0;
        while (gridX < GRID_WIDTH) {
            if (std::rand() % 2 == 0) {
                int length = std::rand() % 4 + 1;
                if (gridX + length <= GRID_WIDTH) {
					index++;
                    blocks.push_back({ {gridX * BLOCK_SIZE, gridY * BLOCK_SIZE, length * BLOCK_SIZE, BLOCK_SIZE}, 0, 0, gridX, gridY, length, index, false, false });
                    gridX += length;
                } else {
                    gridX++;
                }
            } else {
                gridX++;
            }
        }
    }
}

Block *drawBlock(SDL_Renderer* renderer, Block& block) {
	static Block *b;
	if (block.selected == true) b = &block; 

    SDL_Rect rect = { block.gridX * BLOCK_SIZE+1, block.gridY * BLOCK_SIZE+1, block.length * BLOCK_SIZE-1, BLOCK_SIZE-1 };
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255); // Czarny

    SDL_RenderFillRect(renderer, &rect);
    
    return b;

}
void drawSelected(SDL_Renderer *renderer, Block *lastDisplay) {
	
    // zaznaczony element wyświaetl jako ostsni
	SDL_Rect rect = { lastDisplay->gridX * BLOCK_SIZE+1, lastDisplay->gridY * BLOCK_SIZE+1, lastDisplay->length * BLOCK_SIZE-1, BLOCK_SIZE-1 };

	SDL_SetRenderDrawColor(renderer, lastDisplay->selected ? 255 : 0, 0, 0, 255); // Czerwony kolor dla zaznaczonego bloku
	SDL_RenderFillRect(renderer, &rect);
}

void drawGrid(SDL_Renderer* renderer) {
	Block * lastDisplay = nullptr;

    for ( auto& block : blocks) {
		//Block * b = &block;
        lastDisplay = drawBlock(renderer, block);
    }
    if (lastDisplay) drawSelected(renderer, lastDisplay);
    lastDisplay = nullptr;

}

Block* getBlockAt(int x, int y) {
    for (auto& block : blocks) {
		if (x >= block.gridX && x < block.gridX + block.length && y == block.gridY) {
            return &block;
        }
    }
    return nullptr;
}

bool canMove(const Block& block, int direction) {
    if (direction == -1 && block.gridX == 0) return false;
    if (direction == 1 && block.gridX + block.length >= GRID_WIDTH) return false;

    for (const auto& other : blocks) {
        if (other.gridY == block.gridY) {
            if (direction == -1 && other.gridX + other.length == block.gridX) return false;
            if (direction == 1 && other.gridX == block.gridX + block.length) return false;
        }
    }
    return true;
}

void moveBlock(Block& block, int direction) {
	printf ("DIR: %d\n", direction);
    block.gridX += direction;
    block.rect.x += direction * BLOCK_SIZE-1;
    
    printf ("Move: gridX: %d -- rect.x: %d\n", block.gridX, block.rect.x);
}
/*
void moveSelectedElement(Block *selectedBlock, int xrel, int yrel) {
    // Przesuń zaznaczony element o jedno pole w odpowiednim kierunku
    // Na przykład:
    if (xrel > 0) {
		moveBlock(*selectedBlock, 1);
        //selectedElement.x += 1;

    } else if (xrel < 0) {
		moveBlock(*selectedBlock, -1);
       // selectedElement.x -= 1;

    }
    
    if (yrel > 0) {
        //selectedElement.y += 1;
    } else if (yrel < 0) {
        //selectedElement.y -= 1;
    }
    
}
*/

bool freeAtBottom(Block& selected) {

	if (selected.gridY < 7) {
		for (auto& block: blocks) {
			if (block.gridY  == selected.gridY+1) {
				for (int m = 0; m < block.length; m++) {
				printf ("block.gridY: %d --  selected.gridY+1: %d\n", block.gridY +1, selected.gridY);
					for (int n = 0; n < selected.length; n++) {
						if (block.gridX + m == selected.gridX + n) {
							printf ("odnalezione--> (block.gridX)+n: %d  -- (selected.gridX)+n: %d\n", block.gridX + n, selected.gridX + n);
							return false;
						}
					}
				}
			}
		}
		return true;
	}
	return false;
}

int main(int argc, char* argv[]) {
    // Inicjalizacja SDL
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
        std::cerr << "SDL could not initialize! SDL_Error: " << SDL_GetError() << std::endl;
        return 1;
    }

    // Tworzenie okna
    SDL_Window* window = SDL_CreateWindow("Dropdom", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_SHOWN);
    if (window == nullptr) {
        std::cerr << "Window could not be created! SDL_Error: " << SDL_GetError() << std::endl;
        SDL_Quit();
        return 1;
    }

    // Tworzenie renderera
    SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);
    if (renderer == nullptr) {
        std::cerr << "Renderer could not be created! SDL_Error: " << SDL_GetError() << std::endl;
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }

    // Inicjalizacja generatora liczb losowych
    std::srand(std::time(nullptr));

    // Generowanie bloków
    generateBlocks();

    // Główna pętla gry
    //bool quit = false;
    SDL_Event event;
    bool running = true;
	int mouseX;
    int mouseY;
    int gridX;
	int gridY;
    Block* selectedBlock = nullptr;
    Block* underBlock = nullptr;

	std::thread logicThread(gameLogic);

	while (running) {
        while (SDL_PollEvent(&event)) {
            switch (event.type) {
                case SDL_QUIT:
                    running = false;
                    break;

                case SDL_MOUSEBUTTONDOWN:
                    if (event.button.button == SDL_BUTTON_LEFT) {
                        mouseX = event.button.x;
                        mouseY = event.button.y;
						gridX = mouseX / BLOCK_SIZE;
						gridY = mouseY / BLOCK_SIZE;
                        // Sprawdź, który element został kliknięty
                        for (auto& block : blocks) {
                            if (mouseX >= block.rect.x && mouseX <= block.rect.x + block.rect.w &&
                                mouseY >= block.rect.y && mouseY <= block.rect.y + block.rect.h) {
                                block.isDragging = true;
                                //printf("selsected ok\n");
                                if (selectedBlock) {
								selectedBlock->selected = false;
								selectedBlock = nullptr;
								}
                                //selectedBlock = &block;  // Zapamiętaj wybrany element

                                selectedBlock = getBlockAt(gridX, gridY);

								if (selectedBlock) {
									selectedBlock->selected = true;
									//break;
								}
                            }
                        }
                    }
                    break;

                case SDL_MOUSEBUTTONUP:
                    if (event.button.button == SDL_BUTTON_LEFT) {
                        // Zakończ przeciąganie wszystkich elementów
                        for (auto& block: blocks) {
                            block.isDragging = false;
                            block.selected = false;
                        }
						mouseX = event.button.x;
                        mouseY = event.button.y;
						gridX = mouseX / BLOCK_SIZE;
						gridY = mouseY / BLOCK_SIZE;
						underBlock = getBlockAt(gridX, gridY);
						if (underBlock && freeAtBottom(*underBlock)) {
							underBlock->rect.y += BLOCK_SIZE;
							underBlock->gridY += 1;
							}
						underBlock = nullptr;	
                    }
                    break;

                case SDL_MOUSEMOTION:
                    // Przesuń wybrany element
                    if (selectedBlock && selectedBlock->isDragging) {
						selectedBlock->x = event.motion.x; // - selectedBlock->rect.w / 2;
						selectedBlock->y = event.motion.y; // - selectedBlock->rect.h / 2;
						
						if (!moved && selectedBlock->x > mouseX && canMove(*selectedBlock, 1)) {
							std::cout << "Przesuwanie w prawo" << std::endl;
							//elementX += (selectedBlock->rect.x - mouseX); // Przesuń element w prawo
							//moveSelectedElement(selectedBlock, event.motion.xrel, event.motion.yrel);
							moved = true; // Ustaw flagę, aby zapobiec kolejnym przesunięciom

							moveBlock(*selectedBlock, 1);
						} else if (!moved && selectedBlock->x < mouseX && canMove(*selectedBlock, -1)) {
							std::cout << "Przesuwanie w lewo" << std::endl;
							//elementX -= (selectedBlock->rect.x - mouseX); // Przesuń element w lewo
							//moveSelectedElement(selectedBlock, event.motion.xrel, event.motion.yrel);
							moved = true; // Ustaw flagę, aby zapobiec kolejnym przesunięciom

							moveBlock(*selectedBlock, -1);
						}
						
						//mouseX = selectedBlock->rect.x;
                    }
                    break;
            }
        }

        // Czyszczenie ekranu
        SDL_SetRenderDrawColor(renderer, 255, 255, 255, 255); // Biały kolor tła
        SDL_RenderClear(renderer);

        // Rysowanie siatki i bloków
        drawGrid(renderer);

        // Wyświetlenie renderowanego obrazu
        SDL_RenderPresent(renderer);

        // Ograniczenie liczby klatek na sekundę
        SDL_Delay(16); // ~60 FPS
    }
	logicThread.detach();

    // Zwolnienie zasobów
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();

    return 0;
}

